/*
=========================================================================

	FILE:		CP_MemoryRecording.inc
	
				Copyright  1998-2000. Ken Stahlman. All Rights Reserved.
	
	PURPOSE:	Definitions for routines in our CP_MemoryRecording list.
	
	HISTORY:	1st Version - September 2000.
	
	NOTES:		- none -

	VERSION:	Current Revision: $Revision: 2 $
				Latest Change On: $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $ by $Author: kenstah $

=========================================================================
*/

/*
=========================================================================
	FUNCTION:	CP_MemoryRecordingList
	
	PURPOSE:	Copy constructor
=========================================================================
*/
template <class CP_EXPORT T>
CP_MemoryRecordingList<T>::CP_MemoryRecordingList( const CP_MemoryRecordingList& inMemoryRecordingList )
{	
	fSize = inMemoryRecordingList.fSize;
	fData = new T[ fSize ];
	
	std::copy( inMemoryRecordingList.begin(), inMemoryRecordingList.end(), this->begin() );
}

/*
=========================================================================
	FUNCTION:	push_back
	
	PURPOSE:	Add a new element to the back of our list.
=========================================================================
*/
template <class CP_EXPORT T>
void CP_MemoryRecordingList<T>::push_back( T& inElement )
{	
		// allocate space for new element
	T		*newData = new T[ fSize + 1 ];
	
	std::copy( begin(), end(), newData );
	
	delete [] fData;

	fSize = fSize + 1;
	fData = newData;

		// add new element
	fData[ fSize ] = inElement;
}

/*
=========================================================================
	FUNCTION:	erase
	
	PURPOSE:	Remove an element from our list.
=========================================================================
*/
template <class CP_EXPORT T>
typename CP_MemoryRecordingList<T>::iterator CP_MemoryRecordingList<T>::erase( iterator inPosition )
{	
	size_type		n = size_type( end() - inPosition - 1 );
	
	if( n > 0 )
		std::copy( inPosition + 1, end(), inPosition );
	
	--fSize;

	return( inPosition );
}

/*
=========================================================================
	FUNCTION:	operator =
	
	PURPOSE:	Assignment operator
=========================================================================
*/
template <class CP_EXPORT T>
CP_MemoryRecordingList<T>& CP_MemoryRecordingList<T>::operator = ( const CP_MemoryRecordingList& inMemoryRecordingList )
{	
	if( fData != inMemoryRecordingList.fData )
	{
		T		*newData = new T[ inMemoryRecordingList.fSize ];
		
		std::copy( inMemoryRecordingList.begin(), inMemoryRecordingList.end(), newData );
		
		delete [] fData;

		fSize = inMemoryRecordingList.fSize;
		fData = newData;
	}
	
	return( *this );
}

/*
=========================================================================
	FUNCTION:	operator =
	
	PURPOSE:	Assignment operator
=========================================================================
*/
template <class CP_EXPORT T>
bool CP_MemoryRecordingList<T>::operator == ( const CP_MemoryRecordingList& inMemoryRecordingList ) const
{	
	bool		isEqual = false;
	
	if( fSize == inMemoryRecordingList.fSize &&
		std::equal( this->begin(), this->end(), inMemoryRecordingList.begin() )
		isEqual = true;

	return( isEqual );
}

/*
=========================================================================
	FUNCTION:	operator <
	
	PURPOSE:	Less than operator
=========================================================================
*/
template <class CP_EXPORT T>
bool CP_MemoryRecordingList<T>::operator < ( const CP_MemoryRecordingList& inMemoryRecordingList ) const	
{	
	return( std::lexicographical_compare( this->begin(), this->end(),
		inMemoryRecordingList.begin(), inMemoryRecordingList.end() ) );
}

/*
=========================================================================
	FUNCTION:	swap
	
	PURPOSE:	Was our data members
=========================================================================
*/
template <class CP_EXPORT T>
void CP_MemoryRecordingList<T>::swap( CP_MemoryRecordingList<T>& inMemoryRecordingList )
{	
	std::swap( fSize, inMemoryRecordingList.fSize );
	std::swap( fData, inMemoryRecordingList.fData );
}

#if TARGET_CC_METROWERKS
#pragma mark === Local operator new / delete ===
#endif

/*
=========================================================================
	FUNCTION:	operator new
	
	PURPOSE:	Local operator new for creating our memory heap.
=========================================================================
*/
template <class CP_EXPORT T>
void *CP_MemoryRecordingList<T>::operator new( std::size_t size )
{
	void	*ptr = NULL;
	
	#if TARGET_OS_MAC
		ptr = ::NewPtr( size );
	#endif
	
	#if TARGET_OS_WIN32
		ptr = malloc( size );
	#endif
	
	return( ptr );
}

/*
=========================================================================
	FUNCTION:	operator delete
	
	PURPOSE:	Delete our memory heap.
=========================================================================
*/
template <class CP_EXPORT T>
void CP_MemoryRecordingList<T>::operator delete( void * ptr )
{
	if( ptr != nil )
	{
		#if TARGET_OS_MAC
			::DisposePtr( (Ptr) ptr );
		#endif
		
		#if TARGET_OS_WIN32
			free( ptr );
		#endif
	}
}

/*
=========================================================================
	FUNCTION:	operator new[]
	
	PURPOSE:	Local operator new for creating our memory heap.
=========================================================================
*/
template <class CP_EXPORT T>
void *CP_MemoryRecordingList<T>::operator new[]( std::size_t size )
{
	void	*ptr = NULL;
	
	#if TARGET_OS_MAC
		ptr = ::NewPtr( size );
	#endif
	
	#if TARGET_OS_WIN32
		ptr = malloc( size );
	#endif
	
	return( ptr );
}

/*
=========================================================================
	FUNCTION:	operator delete[]
	
	PURPOSE:	Delete our memory heap.
=========================================================================
*/
template <class CP_EXPORT T>
void CP_MemoryRecordingList<T>::operator delete[]( void * ptr )
{
	if( ptr != nil )
	{
		#if TARGET_OS_MAC
			::DisposePtr( (Ptr) ptr );
		#endif
		
		#if TARGET_OS_WIN32
			free( ptr );
		#endif
	}
}


/*
=========================================================================
REVISION HISTORY:

$Log: CP_MemoryRecording.inc,v $
Revision 1.1.1.1  2003/10/31 00:23:47  kenstah
Initial import.

=========================================================================
*/
